//
//  TEBeautyKit.h
//  TEBeautyKit
//
//  Created by tao yue on 2024/1/13.
//

#import <UIKit/UIKit.h>
#import <Foundation/Foundation.h>
#import "TEUIProperty.h"
#import <XMagic/XMagic.h>
@class TEPanelView;

static NSString * const kSandboxPrefix = @"{sandbox}";
static NSString * const kBundlePrefix = @"{bundle}";

typedef void(^OnInitListener)(XMagic * _Nullable api);
typedef void(^callback)(NSInteger authresult, NSString * _Nullable errorMsg);

@protocol TEBeautyKitAIDataListener <NSObject>

- (void)onAIEvent:(id _Nonnull)event;

@end

@protocol TEBeautyKitTipsListener <NSObject>

- (void)onTipsEvent:(id _Nonnull)event;

@end

@interface TEBeautyKit : NSObject

//正在生效的美颜
@property (nonatomic ,strong)NSMutableArray<TESDKParam *>* _Nullable usedSDKParam;

//创建TEBeautyKit对象
+ (void)create:(OnInitListener _Nullable )onInitListener;

//创建TEBeautyKit对象
+ (void)create:(BOOL)isEnableHighPerformance onInitListener:(OnInitListener _Nullable )onInitListener;

//美颜鉴权
+ (void)setTELicense:(NSString *)url key:(NSString *)key completion:(callback _Nullable )completion;

//设置美颜对象
- (void)setXMagicApi:(XMagic *_Nullable)xmagicApi;

//静音
- (void)setMute:(BOOL)isMute;

//开启能力
- (void)setFeatureEnableDisable:(NSString *_Nullable)featureName enable:(BOOL)enable;

//图片美颜
- (UIImage *_Nullable)processUIImage:(UIImage *_Nullable)inputImage
                 imageWidth:(int)imageWidth
                imageHeight:(int)imageHeight
                  needReset:(bool)needReset;

//处理texture
- (YTProcessOutput *_Nullable)processTexture:(int)textureId
         textureWidth:(int)textureWidth
        textureHeight:(int)textureHeight
           withOrigin:(YtLightImageOrigin)origin
      withOrientation:(YtLightDeviceCameraOrientation)orientation;

//处理CVPixelBufferRef
- (YTProcessOutput * _Nullable)processPixelData:(CVPixelBufferRef _Nullable )pixelData
                      pixelDataWidth:(int)pixelDataWidth
                     pixelDataHeight:(int)pixelDataHeight
                          withOrigin:(YtLightImageOrigin)origin
                     withOrientation:(YtLightDeviceCameraOrientation)orientation;

//设置美颜
- (void)setEffect:(TESDKParam *_Nullable)sdkParam;

//设置美颜
- (void)setEffectList:(NSArray<TESDKParam *>*_Nullable)sdkParamList;

//是否开启了美颜增强模式
- (BOOL)isEnableEnhancedMode;

//是否开启美颜增强模式
- (void)enableEnhancedMode:(BOOL)enable;

//获取正在使用的美颜数据
- (NSString *_Nullable)exportInUseSDKParam;

//恢复
- (void)onResume;

//暂停
- (void)onPause;

//销毁
- (void)onDestroy;

//获取当前texture的图片
- (void)exportCurrentTexture:(void (^_Nullable)(UIImage * _Nullable image))callback;

//设置log
- (void)setLogLevel:(YtSDKLoggerLevel)level;

//设置AIDataListener
- (void)setAIDataListener:(id<TEBeautyKitAIDataListener> _Nullable)listener;

//设置TipsListener
- (void)setTipsListener:(id<TEBeautyKitTipsListener> _Nullable)listener;

//保存设置的美颜数据
- (void)saveEffectParam:(TESDKParam *_Nonnull)sdkParam;
//删除某个保存的美颜数据
- (void)deleteEffectParam:(TESDKParam *_Nonnull)sdkParam;
//清空保存的美颜数据
- (void)clearEffectParam;
//获取保存的美颜数据
- (NSMutableArray<TESDKParam *> *_Nonnull)getInUseSDKParamList;

- (void)setTePanelView:(TEPanelView *)tePanelView;

@end
